function fluid(action)
%
% Interfaceprogram for a femlab-simulation in fluid-mechanics
% 
% Authors:  Franz Rieder / Winfried Kernbichler / Ferdinand Schuerrer
% Version: 1.3 
% Date:    January 30, 2002
%          
%
% ==========================================================================
% one should keep this at the beginning of the file
% just change func_name
func_name = 'fluid';                    % function name - same as declaration !!
% ==========================================================================
handle_name = [func_name,'_HANDLE'];  % name of global variable
eval(['global ',handle_name]);        % set it global
if nargin == 0, action = 'initgraph'; end  % set to initgraph without input
if ~strcmp(action,'initgraph'), 
  eval(['HANDLE=',handle_name,';']);  % copy global to local
else
  % one can change this strings
  logo_switch = 1;                  % logo: yes (1), no (0)
  fig_title   = 'Fluid between two rotating cylinders';
  logo_title  = 'Fluid between two rotating cylinders';
  logo_name   = 'Franz Rieder / Winfried Kernbichler / Ferdinand Schuerrer ';
  logo_lv     = 'Multimediale Lehre';
  logo_type   = 'logo_1';           % only this is defined at the moment
  
  min_ri      = 0;                  % minimum inner radius
  max_ri      = 0.9;                % maximum inner radius
  sta_ri      = 0.3;                % standard inner radius
  min_ro      = 1;                  % minimum outer radius
  max_ro      = 5;                  % maximum outer radius
  sta_ro      = 1;                  % standard outer radius
  min_rho     = 0.1;                % minimum density
  max_rho     = 5;                  % maximum density
  sta_rho     = 1;                  % standard density  
  min_eta     = 0;                  % minimum viscosity
  max_eta     = 2;                  % maximum viscosity
  sta_eta     = 0.1;                % standard viscosity
  min_time    = 0.01;               % minimum end-time
  max_time    = 10;                 % maximum end-time
  sta_time    = 1;                  % standard end-time
  min_steps   = 1;                  % minimum number of timesteps
  max_steps   = 100;                % maximum number of timesteps
  sta_steps   = 10;                 % standard number of timesteps
  
  % some colors
  ax_color = [0.9,0.9,0.9];         % default axes color
  bo_color = [0.6,0.6,0.6];         % default border color
  % do not change
  HANDLE = [];                      % empty at the beginning
end
% ==========================================================================
% ==========================================================================
% Here every aspect of the graphics part should be handled
% This contains the definition and the callback routines
switch lower(action)
  % ========================================================================
  % initialization of program
  case 'initgraph'    
    % ----------------------------------------------------------------------
    % preprocessing for initgraph (keep this lines) 
    eval( [func_name,' ''initgraph_pre'''] );
    % ----------------------------------------------------------------------
    
    % ---------------------------------------------------------------------- 
    % definition and creation of graphics objects
    % ----------------------------------------------------------------------

    % ----------------------------------------------------------------------  
    % definition of main figure
    % ----------------------------------------------------------------------  
    F.figure = struct( ...
	'frame',    { [0.0, 0.0, 1.0, 1.0] }, ...
	'geometry', { [1] }, ...
	'closerequestfcn', { 'close_request_main' }, ...
	'renderer', { 'opengl' }, ...
	'name',     { fig_title }, ...
	'tag',      { 'main' } ...
	);
    % --- should be kept ---------------------------------------------------  
    [HANDLE,fig_han] = gui_create(gui_define(F),HANDLE); % creation
    clear F;                                      % not needed any more
    % add function name to tag and closerequestfcn
    % important for finding program related figures and to find function
    gui_add_name(fig_han,func_name,'_','b','tag');
    gui_add_name(fig_han,func_name,' ','b','closerequestfcn');
    % sets necessary userdata for figures
    gui_set_userdata(fig_han, ...
	'function',func_name, ...
	'end','exit');
    figmain_han = fig_han;
    % figure color
    fig_color = get( gui_handle_query(HANDLE.figure,'tag','*main*'), 'color' );

    
    % ----------------------------------------------------------------------  
    % create meta frames
    % ----------------------------------------------------------------------  
    if logo_switch
      logo_y_pos  = 0.9; % y division between logo and rest
    else
      logo_y_pos  = 1.0;
    end
    meta_frame_logo   = [0, logo_y_pos, 1, 1-logo_y_pos]; % logo
    meta_frame_layer1 = [0, 0,          1, logo_y_pos  ]; % layer 1   
    % ----------------------------------------------------------------------  

    % ----------------------------------------------------------------------  
    % create logo
    % ----------------------------------------------------------------------
    if logo_switch
      % --------------------------------------------------------------------  
      logo_string = {logo_title,logo_name,logo_lv};
      logo_parent = figmain_han;
      % here one could use HANDLE instead of HANDLE_LOGO, then one could 
      % refere to all contents within the rest of the program.
      % When using HANDLE_LOGO this graphics part stays seperate. 
      HANDLE_LOGO = [];
      HANDLE_LOGO = gui_create_logo(HANDLE_LOGO, logo_string, ...
	  logo_parent, meta_frame_logo, logo_type);
    end
    % ----------------------------------------------------------------------  
    % ----------------------------------------------------------------------  

    % ---------------------------------------------------------------------- 
    % create main frames at 4 positions
    %  four joined parts with division lines given at 
    %  x: x_div_pos y: y_div_pos
    % ----------------------------------------------------------------------  
    mf = meta_frame_layer1;    % choose meta frame
    x_div_pos = 0.80 * mf(3);  % division line in x direction
    y_div_pos = 0.20 * mf(4);  % division line in y direction

    % frames for lower-left, lower-right, upper-left, upper-right
    ll_frame = [ mf(1),           mf(2),             ...
	  x_div_pos,       y_div_pos ];
    lr_frame = [ x_div_pos,       mf(2) ,            ...
	  mf(3)-x_div_pos, y_div_pos ];
    ul_frame = [ mf(1),           y_div_pos,         ...
	  x_div_pos,       mf(4)-y_div_pos ];
    ur_frame = [ x_div_pos,       y_div_pos,         ...
	  mf(3)-x_div_pos, mf(4)-y_div_pos ];    
    ur_frame1 = [ x_div_pos        ,   y_div_pos+(mf(4)-y_div_pos)*0.35,   ...
	              mf(3)-x_div_pos  ,   (mf(4)-y_div_pos)*0.65              ];	   
    ur_frame2 = [ x_div_pos        ,   y_div_pos,                          ...
             	  mf(3)-x_div_pos  ,   (mf(4)-y_div_pos)*0.35              ];


    % ----------------------------------------------------------------------  
    % definition of some basis axes system for the background of all four
    % parts defined in the meta frame.
    Fbasis.axes = struct( ...
	'geometry', { [1] }, ...
	'xborder',  { [0.0, 0.0] }, ...
	'yborder',  { [0.0, 0.0] }, ...
	'xgap',     { [0.0] }, ...
	'ygap',     { [0.0] }, ...	
	'parent',   { [1] }, ...
	'visible',  { 'on' }, ...
	'box',      { 'on' }, ...
	'xtick',    { [] }, ...
	'ytick',    { [] }, ...
	'color',    { ax_color }, ...
	'xcolor',   { bo_color }, ...
	'ycolor',   { bo_color } ...	
	);
    
    % ----------------------------------------------------------------------  
    % lower left position (containing sliders)
    % ---------------------------------------------------------------------- 
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',ll_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'lower_left',' ','b','tag');            
    
    % definitions of slider values
    geom = [ 1, 1, 1 ;  1, 1, 1 ];
    % typical values
    visible = { 'on',  'on', 'on' ;  'on', 'on', 'on' };
    minval  = { min_ri, min_rho, min_time  ;  min_ro,  min_eta,  min_steps }; % minimum
    maxval  = { max_ri, max_rho, max_time  ;  max_ro,  max_eta,  max_steps }; % maximum
    svalue  = { sta_ri, sta_rho, sta_time  ;  sta_ro,  sta_eta,  sta_steps }; % value for slider
    evalue = svalue;                            % value for edit (string)
    for k = 1:length(svalue(:)), evalue{k} = num2str(svalue{k}); end

    %tag = 'slider';
    cab = 'set_value';
    str = { ...
	'inner_radius','density','end_time'; ...
    'outer_radius','viscosity','timesteps' ...
	};
    tag = lower(str);
    % create variables from string
    var = transpose(flipud(lower(str))); % variables
    % here transpose and flipud is necessary to allow later use with
    % gui_add_userdata 
    % graphics obejects are counted from lower left corner along the 
    % second dimension, arrays and cells from upper left corner along
    % the first dimension.
    % The command lower is used to make sure variable names are lower
    % case.

    % callback action to be performed
    cb_ac = 'setup_main_plot'; cb_no = '';
    cbaction = { cb_no, cb_no, cb_no; cb_no, cb_no, cb_no };
    cbaction = transpose(flipud(lower(cbaction)));
    % what should be changed in the slider
    cb_ch = { 'value','round(##)' }; cb_no = {};
    cbchange_sl = { cb_no, cb_no, cb_no; cb_no, cb_no, cb_ch }; 
    cbchange_sl = transpose(flipud(cbchange_sl));
    % what should be changed in the edit field
    cb_ch = { 'string','num2str((round(str2num(##))))' };	cb_no = {};
    cbchange_ed = { cb_no, cb_no, cb_no; cb_no, cb_no, cb_ch };
    cbchange_ed = transpose(flipud(cbchange_ed));
    % what should be updated when slider changes, see also updatehandle
    update_sl = { { 'value','string','num2str(##)' } };
    update_sl = transpose(flipud(update_sl));
    % what should be updated when edit changes, see also updatehandle    
    update_ed = { { 'string','value','str2num(##)' } };
    update_ed = transpose(flipud(update_ed));

        
    
    Fu.uicontrol = struct( ...
	'frame',         { ll_frame }, ...
	'geometry',      { geom }, ...
	'xborder',       { [0.05, 0.05] }, ...
	'yborder',       { [0.1, 0.3] }, ...
	'xgap',          { [0.05] }, ...
	'ygap',          { [0.3] }, ...
	'xshift',        { [0.0] }, ...	
	'yshift',        { [0.0] }, ...	
	'xmodify',       { 'size' }, ...
	'parent',        { [1] }, ...
	'style',         { 'slider' }, ...
	'min',           { minval }, ...
	'max',           { maxval }, ...
	'value',         { svalue }, ...
	'visible',       { visible }, ...
	'interruptible', { 'off' }, ...
	'fontsize',      { 1 }, ...
	'string',        { str }, ...
	'callback',      { cab }, ...
	'tag',           { tag } ...
	);
    [HANDLE,slider_han] = gui_create(gui_define(Fu),HANDLE);
    % sets variable name in userdata + action and update information
    gui_set_userdata(slider_han,'variable',var); 
    gui_set_userdata(slider_han,'cbaction',cbaction);
    gui_set_userdata(slider_han,'cbchange',cbchange_sl); 
    gui_set_userdata(slider_han,'updateproperty',update_sl); 
    gui_add_name(slider_han,'slider',' ','e','tag');    
    % add edit field shifted above (look for 'yshift')
    Fu.uicontrol = setfield(Fu.uicontrol,'xmodify',             'left');
    Fu.uicontrol = setfield(Fu.uicontrol,'xsize',               0.10);
    Fu.uicontrol = setfield(Fu.uicontrol,'yshift',              0.20);
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'edit');
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'left');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              evalue);    
    Fu.uicontrol = setfield(Fu.uicontrol,'fontsize',            0.60);
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            'edit_check');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,edit_han] = gui_create(gui_define(Fu),HANDLE);

    
    % sets variable name in userdata + action and update information
    gui_set_userdata(edit_han,'variable',var); 
    gui_set_userdata(edit_han,'cbaction',cbaction);
    gui_set_userdata(edit_han,'cbchange',cbchange_ed);
    gui_set_userdata(edit_han,'updateproperty',update_ed);
    gui_set_userdata(edit_han,'callback',[func_name,' ',cab]);
    gui_add_name(edit_han,'slider_edit',' ','e','tag');    
    % add text shifted above 
    Fu.uicontrol = setfield(Fu.uicontrol,'xmodify',             'right');
    Fu.uicontrol = setfield(Fu.uicontrol,'xsize',               0.15);
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'text');
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'right');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              str);    
    Fu.uicontrol = setfield(Fu.uicontrol,'fontsize',            0.8);
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            '');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,text_han] = gui_create(gui_define(Fu),HANDLE);
    gui_add_name(text_han,'slider_text',' ','e','tag');    

    % updatehandle for slider (handles of edit)
    edit_cell = mat2cell(edit_han,1,ones(size(edit_han)));
    gui_set_userdata(slider_han,'updatehandle',edit_cell); 
    % updatehandle for edit (handles of slider)
    slider_cell = mat2cell(slider_han,1,ones(size(slider_han)));
    gui_set_userdata(edit_han,'updatehandle',slider_cell); 
    % adds function name to callback    
    gui_add_name([slider_han,edit_han],func_name,' ','b','callback');
    % adds common name to tag
    gui_add_name([slider_han,edit_han,text_han],'lower_left',' ','b','tag');
    % clears structur Fu    
    clear Fu
        
    % ----------------------------------------------------------------------  
    % lower right position
    % ----------------------------------------------------------------------  
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',lr_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'lower_right',' ','b','tag');            

    % ----------------------------------------------------------------------  
    % define uicontrol pushbutton
    geom = [1,1 ; 1,1];
    str = reshape({ 'Start','Help', 'Stop', 'Exit'},size(geom));
    tag = lower(str);
    cab = lower(str);
    
    F.uicontrol = struct( ...
	'frame',    { lr_frame }, ...
	'geometry', { geom }, ...
	'parent',   { [1] }, ...
	'style',    { 'pushbutton' }, ...
	'visible',  { 'on' }, ...
	'fontsize', { 0.3 }, ...
	'string',   { str }, ...
	'callback', { cab }, ...
	'tag',      { tag } ...
	);
    [HANDLE,newh] = gui_create(gui_define(F),HANDLE); clear F;
    gui_add_name(newh,func_name,' ','b','callback');
    gui_add_name(newh,'lower_right',' ','b','tag');            
    % settings for stop handle
    stop_handle = gui_handle_query(newh,'tag','*stop');
    gui_set_userdata(stop_handle,'stop',0);
    
    % ----------------------------------------------------------------------  
    % upper left position
    % ----------------------------------------------------------------------
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',ul_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);    
    gui_add_name(newh,'upper_left',' ','b','tag');            

    % ----------------------------------------------------------------------  
    % add axes for results
    geom = [ 2,2,1 ; 2,2,1 ];
    tag  = { ...
	    'main_plot','main_plot','femlabplot';    ...
        'main_plot','main_plot','timeplot';    ...
	    };

    F.axes = struct( ...
	'frame',    { ul_frame }, ...
	'geometry', { geom }, ...
	'parent',   { [1] }, ...
	'visible',  { 'on' }, ...
	'box',      { 'off' }, ...
	'tag',      { tag } ...
	);
    [HANDLE,axes_h] = gui_create(gui_define(F),HANDLE);clear F;
    gui_add_name(axes_h,'result','_','b','tag');
    gui_add_name(axes_h,'upper_left',' ','b','tag');
    
    im_ax_han=gui_handle_query(HANDLE.axes,'tag','*timeplot');
    new_flag=0;
    set(im_ax_han,'userdata',new_flag);
    im_ax_han=gui_handle_query(HANDLE.axes,'tag','*main_plot');  % get axes-handle
    pos=get(im_ax_han,'position');
    set(im_ax_han,'position',[pos(1:3) pos(4)*0.95]);

    % ----------------------------------------------------------------------  
    % upper right position
    % ----------------------------------------------------------------------  
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',ur_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'upper_right',' ','b','tag');            

    % pushbutton
    geom = [1 ; 1 ; 1 ; 1; 1; 1];
    str = reshape({ 'Solve','PlotNumber','View','Movie-Export','Save','Load'},size(geom));
    tag = lower(str);
    
    cab1     = 'solve_pde';
    cab2     = 'set_value';
    cab3     = 'view_plot';
    cab4     = 'movie_export';
    cab_save = 'save_data';
    cab_load = 'load_data';
    cab  = reshape({ cab1, cab2, cab3, cab4, cab_save, cab_load },size(geom));
  
    Fpush.uicontrol = struct( ...
    	'frame',    { ur_frame1 }, ...
    	'geometry', { geom }, ...
    	'yborder',  { [0.25,0.05] }, ...
    	'ygap',     { [0.03] } , ...
    	'parent',   { [1] }, ...
    	'style',    { 'pushbutton' }, ...
    	'visible',  { 'on' }, ...
    	'fontsize', { 0.3 }, ...
    	'string',   { str }, ...
    	'callback', { cab }, ...
    	'tag',      { tag } ...
    	);
    [HANDLE,newh] = gui_create(gui_define(Fpush),HANDLE); clear Fpush;
    gui_add_name(newh,func_name,' ','b','callback');
    gui_add_name(newh,'upper_right',' ','b','tag');
    reset_han = gui_handle_query(newh,'tag','*reset');
    set(reset_han,'enable','off')
    % convert the PlotNumber-button to a slider :
    uicontrol_han = HANDLE.uicontrol;   
    pn_han = gui_handle_query(uicontrol_han,'tag','*plotnumber');
    pos=get(pn_han,'Position');  
    set(pn_han,'Style','slider','Position',[pos(1:3) 0.03],'Min',1,'Max',10,'Value',1,'SliderStep',[0.1 1]);   
    cb_ch={ 'value','round(##)' };
    cb_ch=transpose(flipud({cb_ch}));
    var=transpose(flipud(tag(2)));
    gui_set_userdata(pn_han,'variable',var);
    gui_set_userdata(pn_han,'cbaction','');
    gui_set_userdata(pn_han,'cbchange',cb_ch);
    gui_set_userdata(pn_han,'updateproperty','');  
   
    % ----------------------------------------------------------------------  
    % create popup - objects:
    geom = [1 ; 1 ; 1 ];
    
    str = reshape({'initial_values','inner_boundary_values','outer_boundary_values'},size(geom));   
    values = reshape(...
           { 'Fluid rotates clockwise|Fluid does not move ',...
             'Cylinder stopped|Rotation counter-clockwise|Rotation clockwise' ,...
             'Cylinder stopped|Rotation counter-clockwise|Rotation clockwise' },size(geom));
    initial_cond.u      = {  'y'        ,  '0'  };
    initial_cond.v      = {  '-x'       ,  '0'  };   
    inner_bnd_cond.u    = {  '0'        ,  '-y' ,  'y'   };
    inner_bnd_cond.v    = {  '0'        ,  'x'  ,  '-x'  };
    inner_bnd_cond.type = {  'noslip'   ,  'uv' ,  'uv'  };
    outer_bnd_cond.u    = {  '0'        ,  '-y' ,  'y'   };
    outer_bnd_cond.v    = {  '0'        ,  'x'  ,  '-x'  };
    outer_bnd_cond.type = {  'noslip'   ,  'uv' ,  'uv'  };
        
    tag = lower(str);
    var = transpose(flipud(lower(str)));
    cab  = reshape({'' , '' , ''},size(geom));       
    % set the text fields:
    Fpop.uicontrol = struct( ...
    	'frame',    { ur_frame2 }, ...
    	'geometry', { geom }, ...
    	'yborder',  { [0.25,0.05] }, ...
    	'ygap',     { [0.1] } , ...
    	'parent',   { [1] }, ...
    	'style',    { 'text' }, ...
    	'visible',  { 'on' }, ...
    	'fontsize', { 0.35 }, ...
        'horizontalalignment',   {'right'}, ...
        'backgroundcolor', { ax_color }, ...
  	    'string',   { str }, ... 
        'tag',      { tag }, ...
        'xmodify',  { 'size' }     );
    [HANDLE,poptext_han] = gui_create(gui_define(Fpop),HANDLE);

    % add popup-fields shifted above (look for 'yshift)
    Fpop.uicontrol = setfield(Fpop.uicontrol,'xmodify',             'size');
    Fpop.uicontrol = setfield(Fpop.uicontrol,'xsize',               0.10);
    Fpop.uicontrol = setfield(Fpop.uicontrol,'ysize',               0.0);
    Fpop.uicontrol = setfield(Fpop.uicontrol,'horizontalalignment', 'center');
    Fpop.uicontrol = setfield(Fpop.uicontrol,'yshift',              -0.07);
    Fpop.uicontrol = setfield(Fpop.uicontrol,'style',               'popup'); 
    Fpop.uicontrol = setfield(Fpop.uicontrol,'string',              values);    
    Fpop.uicontrol = setfield(Fpop.uicontrol,'fontsize',            0.35);
    Fpop.uicontrol = setfield(Fpop.uicontrol,'callback',            '');   
    [HANDLE,popuph] = gui_create(gui_define(Fpop),HANDLE);    
    gui_set_userdata(popuph,'variable',var);
    
    gui_add_name(popuph,'popup',' ','b','tag');    
    
    % set the conditions:
    handle = gui_handle_query(popuph,'tag','*initial_values');
    gui_set_userdata(handle,'cond',initial_cond);
    handle = gui_handle_query(popuph,'tag','*inner_boundary_values');
    gui_set_userdata(handle,'cond',inner_bnd_cond);
    handle = gui_handle_query(popuph,'tag','*outer_boundary_values');
    gui_set_userdata(handle,'cond',outer_bnd_cond);
    
    
    % set enable-status for buttons:
    uicontrol_han = HANDLE.uicontrol;   
    handle = gui_handle_query(uicontrol_han,'tag','*stop');
    set(handle,'enable','off');
    handle = gui_handle_query(uicontrol_han,'tag','*start');
    set(handle,'enable','off');
    handle = gui_handle_query(uicontrol_han,'tag','*save');
    set(handle,'enable','off');
    handle = gui_handle_query(uicontrol_han,'tag','*movie-export');
    set(handle,'enable','off');
    handle = gui_handle_query(uicontrol_han,'tag','*plotnumber');
    set(handle,'enable','off');
    handle = gui_handle_query(uicontrol_han,'tag','*view');
    set(handle,'enable','off');
    
    
    % ----------------------------------------------------------------------  
    % keep this here to setup first image 
    % ----------------------------------------------------------------------  
    eval( [handle_name,'=HANDLE;'] );
    
    % ----------------------------------------------------------------------  
    % postprocessing for initgraph (keep this lines)
    % ----------------------------------------------------------------------  
    eval( [func_name,' ''initgraph_post'''] );
    % ----------------------------------------------------------------------  
    
    % ----------------------------------------------------------------------  
    % set main figure to visible on (keep this lines)   
    % ----------------------------------------------------------------------  
    fig_han = eval([handle_name,'.figure']);   % get content of figure
    fig_han = gui_handle_query(fig_han,'tag',['*main']);
    set(fig_han,'visible','on');          % set figures to visible
    % ======================================================================
   
  case 'initgraph_pre'               
    % preprocessing for initgraph (keep this lines)    
    % ======================================================================
    gui_default('set') % default values
    % close figures belonging to this program
    fig_han = get(0,'children');   
    fig_han = gui_handle_query(fig_han, ...
	'type','figure','tag',[func_name,'*']);
    delete(fig_han);
    % ======================================================================
  
  case 'initgraph_post'               
    % postprocessing for initgraph (keep this lines)   
    % ======================================================================
    gui_default('remove'); % reset defaults to MATLAB defaults
    % ======================================================================
  
  case 'exit'
  % finishing the program (keep this lines)
  % ========================================================================
    fig_han = eval([handle_name,'.figure']); % get all figures
    fig_han = fig_han(ishandle(fig_han));    % check if exist as handle
    delete(fig_han);                         % closes all figures
    clear('global',handle_name)              % deletes the global variable
    % ======================================================================
  
  case 'close_request_main'  
    % close request main (keep this lines)
    % ======================================================================
    func = gui_get_userdata(gcf,'function'); % function name
    exit = gui_get_userdata(gcf,'end');      % action for exit
    msg = { ...
	['Closing this figure exits program ',func,'.']; ...
	'Do you really want to exit?' ...
	};
    tit = 'Close Request Function';
    selection = questdlg(msg,tit,'Yes','No','Yes');
    switch selection,
      case 'Yes',
	eval( [func,' ',exit] )
      case 'No'
	return
    end
    % ======================================================================
    
  case 'close_request_help'  
    % close request for help (keep this lines)
    % ======================================================================
    func = gui_get_userdata(gcf,'function');
    exit = gui_get_userdata(gcf,'end');
    msg = { ...
	'Closing this figure is not possible.'; ...
	'Do you want to make it invisible?' ...
	};
    tit = 'Close Request Function';
    selection = questdlg(msg,tit,'Yes','No','Yes');
    switch selection,
      case 'Yes',
	eval( [func,' ',exit] )
      case 'No'
	return
    end    
    % ======================================================================
        
  case 'help'
    % calling help
    % ======================================================================
    frame = [0.1, 0.1, 0.8, 0.5];
    [HANDLE,new] = gui_help(HANDLE,frame,func_name);
    if new, eval([handle_name,'=HANDLE;']); end % export to global
    % ======================================================================
  
  case 'edit_check'
    % checks string from edit field
    % ======================================================================
    str    = get(gcbo,'string');
    oldval = get(gcbo,'value');
    minval = get(gcbo,'min');
    maxval = get(gcbo,'max');
    val = str2num(str);
    % handle all error cases
    if ~isempty(val)
      if val<minval
	msg = { 'Value to small!','Value set to minimum.' };
	e_han = msgbox(msg,'Edit Box','error','modal');
	set(gcbo,'string',num2str(minval));
      elseif val>maxval
	msg = { 'Value to large!','Value set to maximum.' };
	e_han = msgbox(msg,'Edit Box','error','modal');
	%e_han = errordlg(msg,'Edit Box','on');
	set(gcbo,'string',num2str(maxval));
      end
    else
      msg = { 'Wrong value!','Value set to old value.' };
      e_han = msgbox(msg,'Edit Box','error','modal');
      %e_han = errordlg(msg,'Edit Box','on');
      if isfield(get(gcbo,'userdata'),'updatehandle')
	updatehandle   = gui_get_userdata(gcbo,'updatehandle');      
	updateproperty = gui_get_userdata(gcbo,'updateproperty');
	oldval = get(updatehandle,updateproperty{2});
      end
      set(gcbo,'string',num2str(oldval));
    end
    
    % do the real callback
    if isfield(get(gcbo,'userdata'),'callback')
      eval( gui_get_userdata(gcbo,'callback') );
    end
    % ======================================================================   
    
  case 'set_value'
    % get values from caller 
    % ======================================================================
   
    e_str  = gui_get_userdata(gcbo,'variable');
    cbaction = gui_get_userdata(gcbo,'cbaction');
    figure_han = get(gcbo,'parent');
    func_name = gui_get_userdata(figure_han,'function');
    
    if isfield(get(gcbo,'userdata'),'cbchange')
      cbchange = gui_get_userdata(gcbo,'cbchange');
      if ~isempty(cbchange)
	prop = cbchange{1};
	chfunc = cbchange{2};
	e_str = [ 'get(gcbo,''',prop,''')' ];
	e_str = strrep(chfunc,'##',e_str);
	e_str = [ 'set(gcbo,''',prop,''',',e_str,');' ];
	eval( e_str );
      end
    end
        
    % do the update of another handle
    if isfield(get(gcbo,'userdata'),'updatehandle')
      updatehandle   = gui_get_userdata(gcbo,'updatehandle');      
      updateproperty = gui_get_userdata(gcbo,'updateproperty');      
      if ~isempty(updatehandle)
	from_prop = updateproperty{1};
	to_prop   = updateproperty{2};
	chfunc    = updateproperty{3};
	e_str = [ 'get(gcbo,''',from_prop,''')' ];
	e_str = strrep(chfunc,'##',e_str);
	e_str = [ 'set(updatehandle,''',to_prop,''',',e_str,');' ];
	eval( e_str );
      end
    end
       
    if ~isempty(cbaction)
      eval( [func_name,' ',cbaction] );
    end
    % ======================================================================

  case 'solve_pde'

    % solve the navier-stokes-equations with femlab
    % ======================================================================
    uicontrol_han = HANDLE.uicontrol;   
    % get variables from slider values      
    data_handle = gui_handle_query(uicontrol_han,'tag','*slider');
    % set all variables
    variables = gui_get_userdata(data_handle,'variable');
    values    = get(data_handle,'value');
    lv = length(values);
    if lv==1
       eval([variables,'=values;']);
    else
      for k = 1:lv
	    value = values{k};
    	var   = variables{k};
    	eval([var,'=value;']);
      end
    end
    % get variables from popup values
    data_handle = gui_handle_query(uicontrol_han,'tag','*popup*');
    % set all variables
    variables = gui_get_userdata(data_handle,'variable');
    values    = get(data_handle,'value');
    lv = length(values);
    if lv==1
      eval([variables,'=values;']);
    else
      for k = 1:lv
	    value = values{k};
    	var   = variables{k};
    	eval([var,'=value;']);
      end
    end     
    
    % get initial/boundary values corresponding to the popup-value:
    
    han = gui_handle_query(uicontrol_han,'tag','*popup initial_values');
    initial_cond = gui_get_userdata(han,'cond');
    han = gui_handle_query(uicontrol_han,'tag','*popup inner_boundary_values');
    inner_bnd_cond = gui_get_userdata(han,'cond');
    han = gui_handle_query(uicontrol_han,'tag','*popup outer_boundary_values');
    outer_bnd_cond = gui_get_userdata(han,'cond');

    u0 = char(initial_cond.u(initial_values));
    v0 = char(initial_cond.v(initial_values));
    ui = char(inner_bnd_cond.u(inner_boundary_values));
    vi = char(inner_bnd_cond.v(inner_boundary_values));
    ti = char(inner_bnd_cond.type(inner_boundary_values));
    uo = char(outer_bnd_cond.u(outer_boundary_values));
    vo = char(outer_bnd_cond.v(outer_boundary_values));
    to = char(outer_bnd_cond.type(outer_boundary_values));

    solution=fluid_fem(inner_radius,outer_radius,density,viscosity,end_time,timesteps,u0,v0,ui,vi,ti,uo,vo,to); % function call to calculate the solution

    solution.axesupdate=1; % axes-range should be updateed for a new solution
    % save solution-structure in property 'userdata' of the main-axes:   
    data_handle = gui_handle_query(HANDLE.axes,'tag','*main_plot');
    set(data_handle,'userdata',solution);     
    % set the flag for a new solution:
    im_ax_han=gui_handle_query(HANDLE.axes,'tag','*timeplot');
    new_flag=1; % because there is a new solution
    set(im_ax_han,'userdata',new_flag);
    % set the plotnumber-slider to the last timestep:
    pn_han = gui_handle_query(HANDLE.uicontrol,'tag','*plotnumber');
    pn_max=length(solution.sol.tlist);
    set(pn_han,'Max',pn_max,'Value',pn_max,'SliderStep',[1/(pn_max-1) 10/(pn_max-1)]);    
    
    % plot the new solution
    eval( [func_name,'(',char(39),'plot',char(39),')'] ) % char(39) = '  

    
    % set enable-status for buttons:
    uicontrol_han = HANDLE.uicontrol;   
    handle = gui_handle_query(uicontrol_han,'tag','*start');
    set(handle,'enable','on');
    handle = gui_handle_query(uicontrol_han,'tag','*save');
    set(handle,'enable','on');
    handle = gui_handle_query(uicontrol_han,'tag','*movie-export');
    set(handle,'enable','on');
    handle = gui_handle_query(uicontrol_han,'tag','*plotnumber');
    set(handle,'enable','on');
    handle = gui_handle_query(uicontrol_han,'tag','*view');
    set(handle,'enable','on');
    
        % ======================================================================

case 'view_plot'

    % view the solution relating to the plotnumber
    data_handle = gui_handle_query(HANDLE.axes,'tag','*main_plot');        
    solution=get(data_handle,'userdata');
    solution.axesupdate=1; % axes should be rescaled for viewing a new timestep
    set(data_handle,'userdata',solution);           
    % plot the chosen timestep:
    eval( [func_name,'(',char(39),'plot',char(39),')'] )   
    
    %===============================================================================================================
    
case 'plot'  

    % get the data from the main-plot axes (from the userdata-property):
    data_handle = gui_handle_query(HANDLE.axes,'tag','*main_plot');
    solution=get(data_handle,'userdata');
    % get the timestep-number
    pn_han = gui_handle_query(HANDLE.uicontrol,'tag','*plotnumber');
    time=get(pn_han,'Value');
      
    % solution.sol.u  is the solution-matrix 
    % rows    : first third of rows: u, second third: v, last third: p 
    % columns : time
    l=length(solution.sol.u)/3; % 1/3 because there are 3 dependent variables (u,v,p)
    x=solution.mesh.p(1,1:l); % x-coordinates from the femlab-mesh
    y=solution.mesh.p(2,1:l); % y-coordinates
    u=solution.sol.u(1:l,time);     
    v=solution.sol.u(l+1:2*l,time); 
    z=sqrt(u.^2+v.^2);   % calculate values for the surface-plot
    % min / max -  values in   x / y / z  - direction :
    xmin=min(x); 
    xmax=max(x);  
    ymin=min(y);
    ymax=max(y);
    zmin=min(z)-0.1*(max(z)-min(z));
    zmax=max(z)*1.1;
    % interpolate to get a rectangular mesh in x-y-plane :
    interp=200; % number of interpolations
    xi = linspace(xmin,xmax,interp);  
    yi = linspace(ymin,0,round(interp/2));   % just go from y=ymin to y=0 to get slice-plot
    [XI,YI] = meshgrid(xi,yi);
    ZIN = griddata(x,y,z,XI,YI);     
    
    % replace the interploated values within the inner cylinder
    r=sqrt(x.^2+y.^2); % get the radius for each point of the femlab-mesh
    RI=sqrt(XI.^2+YI.^2); %get the radius for the interpolated values
    dummy=min(r)*ones(round(interp/2),interp); % create a matrix (same dimensions as RI) with min(r)-values
    factor=(RI>dummy); % creat factor for valid RI-values (1) or unvalid RI-values (0)
    I=find(factor==0); % search indices for wrong RI-values
    factor(I)=nan; % replace each unvalid RI
    ZI=ZIN.*factor;   
    
    % get the values for the line at y=0:
    x3=XI(1,:);
    y3=YI(interp/2,:);
    z3=ZI(interp/2,:);
    %=========================================================================================================================    
    % calculate the vorticity ( w=rot(u) );
    % at first we have to calculate an interpolation-function (we choose polynomial fit with least square condition)
    
    xsp=x3(find(x3>0)); % x3=r; get only the 'right side' of the cylinder
    ysp=z3(find(x3>0));   

    xsp=xsp(find(ysp>-1)); % ysp>0 or ysp=NaN -> with this command we cut off the NaN-values
    ysp=ysp(find(ysp>-1));
    
    xsp=xsp(10:length(xsp)-10);
    ysp=ysp(10:length(ysp)-10);
    
    polycoef=polyfit(xsp,ysp,4); % fit polynomial of order 4
    xint=linspace(xsp(1),xsp(length(xsp)),1*length(xsp)); % interpolation-points
    % create the fit-function with the coefficient (get from polycoef):
    % polyfunc is the fit-function for u=u(r,t) for a determined time
    polyfunc=zeros(1,length(xint));
    for i=1:length(polycoef)
      polyfunc=polyfunc+ polycoef(i).*xint.^(length(polycoef)-i); 
    end
    % create the derivation of the fit-function:   
    derpolyfunc=zeros(1,length(xint));
    for i=1:(length(polycoef)-1)     
      derpolyfunc=derpolyfunc + (length(polycoef)-i).*polycoef(i).*xint.^(length(polycoef)-1-i);
    end
    % calculate the vorticity-function w=rot(u)
    % because we have only u_theta ( and no u_r or u_z ) the rotation is the same as 
    %
    %               1     d                   ->   
    % w = rot(u)=   -  * ---- (r * u_theta) * ez
    %               r     dr
    %
    % and so we get the following:
    
    w = polyfunc./xint + derpolyfunc;
    
    %=========================================================================================================================    
    femlab_ax_han=gui_handle_query(HANDLE.axes,'tag','*femlabplot'); 
    axes(femlab_ax_han);
    miny=min([min(polyfunc),min(w)]);
    maxy=max([max(polyfunc),max(w)]);
    miny=miny-0.05*(maxy-miny);
    maxy=maxy+0.05*(maxy-miny);   
    hold on
    child = get(femlab_ax_han,'Children');  %look for plot/surface in these axes
    if length(child)>0
        delete(child);
    end      
    plot(xint,polyfunc,'b-',xint,w,'r-');
    axis(femlab_ax_han,[ 0 max(xint) miny maxy ]);  
    hold off;
    %=========================================================================================================================
    time_ax_han=gui_handle_query(HANDLE.axes,'tag','*timeplot');
    new_flag=get(time_ax_han,'userdata');
    if new_flag==1
      save_tag=get(time_ax_han,'tag');        
      axes(time_ax_han);
      l=length(solution.sol.u)/3; % 1/3 because there are 3 dependent variables (u,v,p)
      timelist=solution.sol.tlist; % get the time-list
      r_max=[];
      z_max=[];
      for i=1:length(timelist) 
        u=solution.sol.u(1:l,i); % get velocity-values (x-direction) for the i-th timestep
        v=solution.sol.u(l+1:2*l,i); % get velocity-values (y-direction) for the i-th timestep
        z=sqrt(u.^2+v.^2);   % calculate values for the surface-plot
        [zmax,zmax_index]=max(z); % get max-value and index of the maximum
        z_max=[z_max,zmax];
        r_max=[r_max,sqrt(solution.mesh.p(1,zmax_index)^2+solution.mesh.p(2,zmax_index)^2)]; % radius for maximum speed
      end
      z_max=z_max/max(z_max); % normalization -> maximum-value = 1
      r_max=r_max/max(r_max); % normalization -> maximum-value = 1
      [timeindex_for_max_speed,max_speed]=max(z_max);
      plot(timelist,r_max,'b',timelist,z_max,'r','userdata',timeindex_for_max_speed,'tag','time_plot');
      line([0,0],[0,0],'LineStyle',':','Color',[0.3 0.3 0.3],'tag','time_line');   
      axis(time_ax_han,[0,timelist(length(timelist)),0,1.1]);
      xl=get(time_ax_han,'xlabel');
      set(xl,'String','time [s]')
      yl=get(time_ax_han,'ylabel');
      set(yl,'String','r / ro    u / uo');
      new_flag=0;
      set(time_ax_han,'userdata',new_flag,'tag',save_tag);
    end;
    % set the dotted line in the time-plot :
    line_han=get(time_ax_han,'children');
    line_han=gui_handle_query(line_han,'tag','time_line');
    set(line_han,'xdata',[solution.sol.tlist(time),solution.sol.tlist(time)],'ydata',[0,1.1]);    
    %========================================================================================================================= 
    main_ax_han=gui_handle_query(HANDLE.axes,'tag','*main_plot');  % get axes-handle
    axes(main_ax_han);   %set current axes
    hold on;
    child = get(main_ax_han,'Children');  %look for plot/surface in these axes
    if length(child)==0
       save_tag=get(main_ax_han,'tag');
       surf_han=surf([]);  % create surface-plot
       set(surf_han,'Parent',main_ax_han); % link the surface-plot to the parent-axes  
       plot3_han=plot3(x3,y3,z3);       
       set(plot3_han,'Parent',main_ax_han,'tag','blackline'); % link the 3D-Line-plot to the parent-axes
       set(main_ax_han,'tag',save_tag);    
       view([-0.5*(xmax),0.6*(ymax),0.2]); % set viewpoint
       shading(main_ax_han,'interp'); % set color-mapping property
    end
    child=get(main_ax_han,'Children');
    set(child(2),'XDATA',XI,'YDATA',YI,'ZDATA',ZI,'CData',ZI,'FaceColor','interp');      
    set(child(1),'XDATA',x3,'YDATA',y3,'ZDATA',z3,'Color',[0 0 0]);         
    title(['velocity field at ',num2str(solution.sol.tlist(time)),' s']);
    % the next lines are necessary to rescale the axes for solve/load/view and not rescale for start (movie):
    if solution.axesupdate==1
      axis([xmin xmax ymin ymax zmin zmax]); 
    else 
      time_ax_han=gui_handle_query(HANDLE.axes,'tag','*timeplot');
      plot_han=get(time_ax_han,'children');
      plot_han=gui_handle_query(plot_han,'tag','time_plot');
      time_index=get(plot_han(1),'userdata'); % get the time-index (=plotnumber), where the velocity has it's maximum
      if time==time_index
        axis([xmin xmax ymin ymax zmin zmax]); % set axes-limits to the maximum value    
      end    
    end    
    hold off;    
    drawnow;
    % ======================================================================  

  case 'movie_export'  
    % play a "movie" and save the recorded frames
    % ======================================================================  
    
    % find handle of stop button and of buttons to be disabled
    uicontrol_han = HANDLE.uicontrol;   
    stop_han = gui_handle_query(uicontrol_han,'tag','*stop');       
    disable_han = uicontrol_han(uicontrol_han~=stop_han);
    enable_status = {get(disable_han,'enable')};
    gui_set_userdata(gcbo,'enable_status',enable_status);        
    % disable buttons and sliders, enable stop button
    set(disable_han,'enable','off');
    set(stop_han,'enable','on');

    main_plot_handle = gui_handle_query(HANDLE.axes,'tag','*main_plot');        
    solution=get(main_plot_handle,'userdata');
    solution.axesupdate=0; % axes-range should not be updated for the next timestep
    set(main_plot_handle,'userdata',solution);           
  
    % get the handle of the plotnumber-slider: 
    pn_han = gui_handle_query(HANDLE.uicontrol,'tag','*plotnumber');
    pn_max=get(pn_han,'max');    

    [newfile,newpath] = uiputfile('*.avi','save movie-file');
    ok_flag=0;
    if isequal(newfile,0)|isequal(newpath,0)
       disp('Movie not saved')
    else
       disp(['File ', newpath, newfile, ' has been saved.'])
       ok_flag=1;
    end
    if ok_flag==1
      mov=avifile([newpath,newfile],'QUALITY',100,'COMPRESSION','None'); % none-compression is necessary for unix
      % get the regiion of the axes and a little border :    
      fig_han=gui_handle_query(HANDLE.figure,'tag','*main*'); % get figure-handle
      size=get(fig_han,'position');
      pos=get(main_plot_handle,'position');
      breite=size(3)*pos(3);
      hohe=size(4)*pos(4);
      rect = [ -round(breite/10) , ...
               -round(hohe/10)   , ...
               round(breite*1.18) , ...
               round(hohe*1.20)   ];
    
      for i=1:pn_max
        set(pn_han,'Value',i);            
        % plot the new solution
        eval( [func_name,'(',char(39),'plot',char(39),');'] )

        m(i)=getframe(main_plot_handle,rect);
        mov = addframe(mov,m(i));
        if (gui_get_userdata(stop_han,'stop'))==1 
          break;
        end    
      end;  
      mov=close(mov);    
      % movie(m);
    end    
    % enable buttons, disable stop button     
    gui_set_userdata(stop_han,'stop',0);
    set(stop_han,'enable','off');
    enable_status = gui_get_userdata(gcbo,'enable_status');
    for k = 1:length(disable_han)
      set(disable_han(k),'enable',enable_status{k});
    end
    % ======================================================================
    
    
  case 'start'
    % play a "movie" - plot all timesteps
    % ======================================================================  
    
    % find handle of stop button and of buttons to be disabled
    uicontrol_han = HANDLE.uicontrol;   
    stop_han = gui_handle_query(uicontrol_han,'tag','*stop');    
    disable_han = uicontrol_han(uicontrol_han~=stop_han);
    enable_status = {get(disable_han,'enable')};
    gui_set_userdata(gcbo,'enable_status',enable_status);        
    % disable buttons and sliders, enable stop button
    set(disable_han,'enable','off');
    set(stop_han,'enable','on');

    data_handle = gui_handle_query(HANDLE.axes,'tag','*main_plot');        
    solution=get(data_handle,'userdata');
    solution.axesupdate=0; % axes-range should not be updated for the next timestep
    set(data_handle,'userdata',solution);           
  
    % get the handle of the plotnumber-slider: 
    pn_han = gui_handle_query(HANDLE.uicontrol,'tag','*plotnumber');
    pn_max=get(pn_han,'max');
    for i=1:pn_max
      set(pn_han,'Value',i);            
      % plot the new solution
      eval( [func_name,'(',char(39),'plot',char(39),');'] )
      if (gui_get_userdata(stop_han,'stop'))==1 
        break;
      end    
    end;  

    % enable buttons, disable stop button     
    gui_set_userdata(stop_han,'stop',0);
    set(stop_han,'enable','off');
    enable_status = gui_get_userdata(gcbo,'enable_status');
    for k = 1:length(disable_han)
      set(disable_han(k),'enable',enable_status{k});
    end
    % ======================================================================

  case 'stop'
    % stop
    % ======================================================================
    stop_han = gui_handle_query(HANDLE.uicontrol,'tag','*stop');    
    gui_set_userdata(stop_han,'stop',1);
    % ======================================================================
    
  case 'save_data'
    % saves data to a file
    % ======================================================================
    [newfile,newpath] = uiputfile('*.dat','Save file name');
    ok_flag=0;
    if isequal(newfile,0)|isequal(newpath,0)
       disp('Data not saved')
    else
       disp(['File ', newpath, newfile, ' has been saved.'])
       ok_flag=1;
    end
    if ok_flag==1
      data_handle = gui_handle_query(HANDLE.axes,'tag','*main_plot');
      solution=get(data_handle,'userdata');            
      save([newpath,newfile],'solution');
    end   
    
    
    
  case 'load_data'
    % loades data from a file
    % ======================================================================
    [newfile,newpath] = uigetfile('*.dat','Select a file');
    fname = [newpath,newfile];   
    
    try, load(fname,'-mat');
    catch, return; end
   
    if exist('solution')
       data_handle = gui_handle_query(HANDLE.axes,'tag','*main_plot');        
       solution.axesupdate=1; % axes-range should be updateed for a loaded solution      
       set(data_handle,'userdata',solution);           
       % solution.sol   % view results !!!!!!!!!!!!
       % set the plotnumber-slider to the last timestep:
       pn_han = gui_handle_query(HANDLE.uicontrol,'tag','*plotnumber');
       pn_max=length(solution.sol.tlist);
       set(pn_han,'Max',pn_max,'Value',pn_max,'SliderStep',[1/(pn_max-1) 10/(pn_max-1)]);
       % set the flag for a new solution
       im_ax_han=gui_handle_query(HANDLE.axes,'tag','*timeplot');
       new_flag=1;
       set(im_ax_han,'userdata',new_flag);
       % plot the loaded solution
       eval( [func_name,'(',char(39),'plot',char(39),')'] )          
       % set enable-status for buttons:
       uicontrol_han = HANDLE.uicontrol;   
       handle = gui_handle_query(uicontrol_han,'tag','*start');
       set(handle,'enable','on');
       handle = gui_handle_query(uicontrol_han,'tag','*save');
       set(handle,'enable','on');
       handle = gui_handle_query(uicontrol_han,'tag','*movie-export');
       set(handle,'enable','on');
       handle = gui_handle_query(uicontrol_han,'tag','*plotnumber');
       set(handle,'enable','on');
       handle = gui_handle_query(uicontrol_han,'tag','*view');
       set(handle,'enable','on');              
    end

   
  otherwise
    % ======================================================================
    % warnings if callback routine is not defined (keep this at the end)    
    % ======================================================================
    disp(['Callback not defined: ',action]);
    % ======================================================================

end









