function fluid_fem=fluid_fem(ri,ro,rho,eta,et,ts,u0,v0,ui,vi,bnd_type_i,uo,vo,bnd_type_o);

% FEMLAB Model M-file

% fluid between two rotating cylinders (ri/ro)
% 17.12.2001
% Rieder Franz

flag=0;
if ri>0
   flag=1;  
end
% initial values:
pr0  =  '1';
% boundary conditions:
pr   =  '1';

if flag
   boundary_conditions =  [1 2 3 4 5 6 7 8];
   initial_conditions  =  [1 2 3 4 5 6 7 8];
   bnd_u               =  {uo,uo,ui,ui,uo,ui,ui,uo};
   bnd_v               =  {vo,vo,vi,vi,vo,vi,vi,vo};
   bnd_type            =  {bnd_type_o,bnd_type_o,bnd_type_i,bnd_type_i,bnd_type_o,bnd_type_i,bnd_type_i,bnd_type_o};
else
   boundary_conditions  = [1 2 3 4];
   initial_conditions   = [1 2 3 4];
   bnd_u                = {uo};
   bnd_v                = {vo};
   bnd_type             = {bnd_type_o};
end

rho=num2str(rho);
eta=num2str(eta);

% Space dimensions
fem.sdim={'x','y'};

% Geometry
clear s c p
if flag
   g1=circ2(0,0,ro);
   g2=circ2(0,0,ri);
   Gebiet=g1-g2;
else 
   Gebiet=circ2(0,0,ro); 
end 
objs={Gebiet};
names={'Gebiet'};
s.objs=objs;
s.name=names;

objs={};
names={};
c.objs=objs;
c.name=names;

objs={};
names={};
p.objs=objs;
p.name=names;

drawstruct=struct('s',s,'c',c,'p',p);
fem.draw=drawstruct;
fem.geom=geomcsg(fem);

% Recorded command sequence

clear appl

% Application mode 1
appl.mode       =   'flpdens2d(''dim'',{''u'',''v'',''p''},''sdim'',{''x'',''y''},''submode'',''std'',''tdiff'',''on'')';
appl.dim        =   {'u','v','p'};
appl.form       =   'general';
appl.border     =   'on';
appl.name       =   'ns';
appl.var        =   {};
appl.assign     =   {'Fx';'Fx';'Fy';'Fy';'Kx';'Kx';'Ky';'Ky';'U';'U';'V';'V';'eta';'eta';'rho';'rho'};
appl.equ.rho    =   {rho};
appl.equ.eta    =   {eta};
appl.equ.Fx     =   {'0'};
appl.equ.Fy     =   {'0'};
appl.equ.ind    =   1;
appl.usage      =   1;
appl.bnd.u      =   bnd_u;
appl.bnd.v      =   bnd_v;
appl.bnd.p      =   {pr};
appl.bnd.type   =   bnd_type;
appl.bnd.ind    =   boundary_conditions;
appl.init.sd    =   {{{u0},{v0},{pr0}}};
appl.init.ind   =   1;

fem.appl=appl;

% Boundary  conditions
clear bnd
bnd.q   =   {{{'0'},{'0'},{'0'};{'0'},{'0'},{'0'};{'0'},{'0'},{'0'}}};
bnd.g   =   {{{'0'};{'0'};{'0'}}};
bnd.h   =   {{{'1'},{'0'},{'0'};{'0'},{'1'},{'0'};{'0'},{'0'},{'0'}}};
bnd.r   =   {{{'-u'};{'-v'};{'0'}}};
bnd.var =   {'Kx','-nga1','Ky','-nga2'};
bnd.ind =   initial_conditions;
fem.bnd =   bnd;

% PDE coefficients

clear equ
equ.da  =   {{{'1'},{'0'},{'0'};{'0'},{'1'},{'0'};{'0'},{'0'},{'0'}}};
equ.c   =   {{{'0.02' , '0'    ; '0' , '0.01'} , {'0'    , '0' ; '0.01' , '0'   } , {'0' , '0' ; '0' , '0'};...
              {'0'    , '0.01' ; '0' , '0'   } , {'0.01' , '0' ; '0'    , '0.02'} , {'0' , '0' ; '0' , '0'};...
              {'0'    , '0'    ; '0' , '0'   } , {'0'    , '0' ; '0'    , '0'   } , {'0' , '0' ; '0' , '0'}}};
equ.al  =   {{{'0';'0'},{'0';'0'},{'0';'0'};{'0';'0'},{'0';'0'},{'0';'0'};{'0';'0'},{'0';'0'},{'0';'0'}}};
equ.ga  =   {{{'-0.02.*ux';'-0.01.*(uy+vx)'};{'-0.01.*(vx+uy)';'-0.02.*vy'};{'0';'0'}}};
equ.be  =   {{{'u';'v'},{'0';'0'},{'1';'0'};{'0';'0'},{'u';'v'},{'0';'1'};{'1';'0'},{'0';'1'},{'0';'0'}}};
equ.a   =   {{{'ux'},{'uy'},{'0'};{'vx'},{'vy'},{'0'};{'0'},{'0'},{'0'}}};
equ.f   =   {{{'-(u.*ux+v.*uy+px)'};{'-(u.*vx+v.*vy+py)'};{'-(ux+vy)'}}};
equ.var =   {'U','sqrt(u.^2+v.^2)','V','vx-uy','rho',{'1'},'eta',{'0.01'},'Fx',{'0'},'Fy',{'0'}};
equ.ind =   1;
fem.equ =   equ;

fem.mesh  =  meshinit(fem,'hmax',0.2);
fem.mesh  =  meshrefine(fem);

fem     =  multiphysics(fem);
fem.mat =  'flisop2p1';

fem.sol=femtime(fem,'report','off','nullfun','flspnull','tlist',linspace(0,et,ts),'Atol',1E-0,'Rtol',1E-0,'Ode','ode23s'); % for use with fluid.m

fluid_fem=fem;




