function handle_out = gui_handle_query(handle_in,varargin)
%
% Looks for handles with specific properties
%
% handle_in:  vector of handles or structure of handles
% handle_out: vector of handles or structure of handles 
%             with specific properties
%
% output has same type as input
%
% properties can be given as
%  1) list of properties and values
%     'prop1',val1,'prop2',val2, .....
%  2) structure of properties and values
%     struct( 'prop1',val1,'prop2',val2, .....)
%
%  Properties must be matlab object properties which can be abrevated
%  according to matlab rules.
%
%  For values one can use '*' as a wildcard at the beginning and/or end 
%  of the string.
% 
% Program should not crash with input of wrong properties.
% 
% Author:  Winfried Kernbichler
% Version: 1.0 
% Date:    October 17, 2001
% Status:  Workes very well in all tests at the moment
% Missing: Check of input

% convert structur to vector for easy use
if isstruct(handle_in)
  hin = gui_handle_convert(handle_in);
else
  hin = handle_in;
end

% create cells of properrties and values
if length(varargin)==1 & isstruct(varargin{1})
  S = varargin{1};
  prop  = fieldnames(S);
  value = struct2cell(S); 
else
  props = varargin(1:2:end);
  value = varargin(2:2:end);
end

hout = hin;
for k = 1:length(props) % cycle through properties
  prop = props{k};
  val  = value{k};
  lval_b = length(val);
  if ischar(val)        % look for wildcard *
    wild = findstr(val,'*');
    val(wild) = [];
  end
  lval_a = length(val);  
  h = hout;
  hout = [];  
  for l = 1:length(h) % cycle through handles
    err = 0;
    try,   pval = get(h(l),prop);   % property value
    catch, err=1; end
    if ~err
      lpval = length(pval);
      if ischar(prop) & ischar(val) % both are strings
	if isempty(wild)
	  if strcmp(pval,val), hout = [hout,h(l)]; end
	else
	  fstr = findstr(pval,val);
	  if ~isempty(fstr)              % handle wildcards
	    if all(wild==[1])            % * at the beginning
	      if fstr(end)+lval_a-1==lpval, hout = [hout,h(l)]; end
	    elseif all(wild==[lval_b])   % * at the end 
	      if fstr(1)==1, hout = [hout,h(l)]; end
	    elseif all(wild==[1,lval_b]) % * at beginning and end 
	      hout = [hout,h(l)];
	    end
	  end
	end
      else % no strings
	if pval==val, hout = [hout,h(l)]; end
      end
    end
  end
end

% return to structure if input was structure
if isstruct(handle_in)
  handle_out = gui_handle_convert(hout);
else
  handle_out = hout;  
end
