% [HANDLE,new] = gui_help(HANDLE,frame,func_name)
%
% display content of help file (funcname.hlp) in a special window
% (C) Winfried Kernbichler, Klaus Lichtenegger, 2001

function [HANDLE,new] = gui_help(HANDLE,frame,func_name)
  
% creates or opens help figure
fig_han = gui_handle_query(HANDLE.figure,'tag','*help*');

% if help figure doesn't exist
if isempty(fig_han)
  
  font = 12; % font size
  
  % make new help figure
  new = 1;
  gui_default('set');
  F.figure = struct( ...
      'frame',    { frame }, ...
      'geometry', { [1] }, ...
      'closerequestfcn', { 'close_request_help' }, ...
      'visible',  { 'off' }, ...
      'tag',      { 'help' }, ...
      'Name',     { 'help window' }, ...
      'Units',    { 'normalized' }, ...
      'Menubar',  { 'none' }, ...
      'Position', { [.1 .1 .8 .8] } ...
      );
  [HANDLE,fig_han] = gui_create(gui_define(F),HANDLE);
  clear F;                                 % not needed any more
  gui_add_name(fig_han,func_name,'_','b','tag');    
  gui_add_name(fig_han,func_name,' ','b','closerequestfcn');
  gui_set_userdata(fig_han,'function',func_name,'end','help');
  gui_default('remove');

  % create static text:
  stat.uicontrol = struct( ...
	'geometry', { [1] }, ...
	'parent',   { fig_han }, ...
	'style',    { 'text' }, ...
	'string',   { 'help loading from file' }, ...
	'tag',      { 'helptext' }, ...
	'Units',    { 'normalized' }, ...
	'Position', { [.1 .1 .77 .8] }, ...
	'HorizontalAlignment', { 'left' }, ...
        'Fontsize', { font } ...
	);
   [HANDLE, help_txt] = gui_create(gui_define(stat), HANDLE); clear stat;
    
   helpline = 38;  % number of display lines (not a perfect solution...)

   % create pushbutton to close help window:	    
   hend.uicontrol = struct( ...
	'geometry', { [1] }, ...
	'parent',   { fig_han }, ...
	'style',    { 'pushbutton' }, ...
	'string',   { 'close help window' }, ...
	'tag',      { 'closehlp' }, ...
	'Units',    { 'normalized' }, ...
	'Position', { [.8 .02 .15 .06] }, ...
        'Callback', { ['gui_helphandle e', func_name] } ...
	);
   [HANDLE, help_end] = gui_create(gui_define(hend), HANDLE); clear hend;

   % read helpfile as cell array, determine actual size:
   helptext = wstextread([func_name, '.hlp']);
   helpsize  = size(helptext, 1);

   % merge lines:
   help2(1)=helptext(1); index = 1; 
   for I=2:helpsize
     zeile = cell2mat(helptext(I));
     if isempty(zeile)
       help2(index+1) = {[]};
       index = index + 2;
       help2(index) = {''}; % Leerzeichen am Absatzanfang
     else
       help2(index) = {[cell2mat(help2(index)), zeile, ' ']};
     end % if
   end % for I
   
   % wrap helptext, calculate number of lines: 
   helptext = textwrap(help_txt, help2);
   helpsize  = size(helptext, 1);

   % store helptext in application data
   setappdata(help_txt, 'helptext', helptext);
   setappdata(help_txt, 'helpsize', helpsize);
   setappdata(help_txt, 'helpline', helpline);

   % display helptext:
   set(help_txt, 'String', helptext);

   % if not all lines can be displayed
   if helpsize > helpline
     
      % create slider
      hsld.uicontrol = struct( ...
	  'geometry', { [1] }, ...
	  'parent',   { fig_han }, ...
	  'style',    { 'slider' }, ...
	  'min',      { 1 }, ...
	  'max',      { helpsize-helpline+1 }, ...
          'value',    { helpsize-helpline+1 }, ...
	  'visible',  { 'on' }, ...
	  'tag',      { 'hilfslaufleiste' }, ...
	  'Units',    { 'normalized' }, ...
	  'Position', { [.87 .1 .03 .8] }, ...
	  'Callback', { ['gui_helphandle s', func_name] } ...
	  );
    
      [HANDLE, help_sld] = gui_create(gui_define(hsld), HANDLE); clear hsld;
       
   end %if helpsize > helplines

else % if help figure already exists
  new = 0;
end % if

% get handle of help-button
uic_han = HANDLE.uicontrol;
uic_han = gui_handle_query(uic_han,'style','pushbutton','tag','*help*');

% makes help figure visible and changes text on button
str = lower(get(uic_han,'string'));
if isempty(findstr(str,'off'))
  set(fig_han,'visible','on');
  gui_add_name(uic_han,'off',' ','e','string');
else
  set(fig_han,'visible','off');
  gui_remove_name(uic_han,'off',' ','e','string');
end






