function [rmat] = percont(mat,nlines,direction)
%
% Periodic continuation of arrays
%
% Input:   mat       - 2-dim array
%          nlines    - number of lines
%          direction - direction of continuation
%
% Output:  rmat      - resulting array
%
% example: rmat = percont(mat,1,1)
%           periodic continuation with 1 line in dimension 1
%          rmat = percont(mat,1,[1,2])
%           periodic continuation with 1 line in dimension 1 and 2
%          rmat = percont(mat,[1,2],[1,2])
%           periodic continuation with 1 line in dimension 1 and 2 lines in
%           dimension 2
%
% Author:  Winfried Kernbichler
% Version: 1.0 
% Date:    October 26, 2001
% Status:  Works very fast

dir = 0;
if nargin<2, nlines=1; end
if nargin<3, dir=1; end

sm = size(mat);
dm = ndims(mat);
    
if dir
    rvec = ones(1:dm)*3;
    nl = ones(1,dm)*nlines;
else
    rvec = ones(1:dm);
    if direction >= 1 & direction <= dm
        rvec(direction) = 3;
        nl = zeros(1,dm);
        nl(direction)=nlines;
    else
        error('Wrong direction');
    end
end

if prod(nl<=sm)
    cutstr='';
    for k = 1:dm
        if rvec(k)==3
            cutstr=[cutstr,',', ...
                    num2str(sm(k)),'+1-',num2str(nl(k)),':', ...
                    num2str(2*sm(k)),'+',num2str(nl(k))];
        else
            cutstr=[cutstr,',1:',num2str(sm(k))];               
        end
    end
    cutstr(1) = [];
    rmat = cut(repmat(mat,rvec),cutstr);
else
    error('Number of continuation lines to big');
end